@@ -132,25 +132,20 @@ delegate_fill (pixman_implementation_t *imp,
 }
 
 pixman_implementation_t *
-_pixman_implementation_create (pixman_implementation_t *toplevel,
-			       pixman_implementation_t *delegate)
+_pixman_implementation_create (pixman_implementation_t *delegate)
 {
     pixman_implementation_t *imp = malloc (sizeof (pixman_implementation_t));
+    pixman_implementation_t *d;
     int i;
     
     if (!imp)
 	return NULL;
-    
-    if (toplevel)
-	imp->toplevel = toplevel;
-    else
-	imp->toplevel = imp;
-    
-    if (delegate)
-	delegate->toplevel = imp->toplevel;
-    
+
+    /* Make sure the whole delegate chain has the right toplevel */
     imp->delegate = delegate;
-    
+    for (d = imp; d != NULL; d = d->delegate)
+	d->toplevel = imp;
+
     /* Fill out function pointers with ones that just delegate
      */
     imp->composite = delegate_composite;
